/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "pokergame.h"
#include "../../libs.h"
#include "pokergameassets.h"
#include "../../poker/poker.h"
#include "../../vn/conversation/talk.h"
#include "../../vn/vnscene.h"
#include "../../util/array.h"
#include "pokerui.h"
#include "pokerworld.h"
#include "pokergameaction.h"

/**
 * Initializes the game state for the poker game.
 * 
 * @param game Game to initialize.
 * @returns True if successful, otherwise false.
 */
bool gameInstanceInit(pokergame_t *game);

/**
 * Updates the poker game instance.
 * @param game Poker game to update for.
 */
void gameInstanceUpdate(pokergame_t *game);

/**
 * Disposes a previously initialized poker game instance.
 * @param game Game to dispose.
 */
void gameInstanceDispose(pokergame_t *game);

/**
 * Restacks the poker game's stack.
 * 
 * @param game Poker game to restack
 */
void pokerGameQueueRestack(pokergame_t *game);