/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../../libs.h"
#include "../../display/shader.h"
#include "../../display/primitive.h"
#include "../../display/primitives/skywall.h"
#include "../../vn/vnscene.h"
#include "../../vn/vncharacter.h"
#include "../../poker/player.h"
#include "../../poker/dealer.h"
#include "pokergameassets.h"


#define POKER_WORLD_SEAT_DISTANCE -1
#define POKER_WORLD_SEAT_ROTATION(n) (n * mathDeg2Rad(45.0f))

#define POKER_WORLD_SEAT_POSITION_X(n) ( \
  POKER_WORLD_SEAT_DISTANCE * (float)sin(POKER_WORLD_SEAT_ROTATION(n)) \
)
#define POKER_WORLD_SEAT_POSITION_Y -0.2f
#define POKER_WORLD_SEAT_POSITION_Z(n) ( \
  POKER_WORLD_SEAT_DISTANCE * (float)cos(POKER_WORLD_SEAT_ROTATION(n)) \
)

#define POKER_GAME_SEAT_COUNT 8
#define POKER_GAME_SEAT_FOR_PLAYER(p) (p - (POKER_PLAYER_COUNT/2))
#define POKER_GAME_SEAT_DEALER POKER_GAME_SEAT_FOR_PLAYER(POKER_DEALER_INDEX)

#define pokerGameSeatFromIndex(i) (\
  i == POKER_DEALER_INDEX ? \
  POKER_GAME_SEAT_DEALER : \
  POKER_GAME_SEAT_FOR_PLAYER(i) \
)

#define POKER_GAME_PENNY_BASE_WIDTH 1000
#define POKER_GAME_PENNY_BASE_HEIGHT 1920
#define POKER_GAME_PENNY_FACE_X 367
#define POKER_GAME_PENNY_FACE_Y 256
#define POKER_GAME_PENNY_FACE_WIDTH 280
#define POKER_GAME_PENNY_FACE_HEIGHT 280


typedef struct {
  primitive_t skywall;
} pokerworld_t;


/**
 * Initialize the poker renderer.
 * 
 * @param world Pointer to the world to initialize.
 * @param scene Visual Novel Engine scene to initialize.
 * @param assets Assets to use for initialization.
 */
void pokerWorldInit(
  pokerworld_t *world,
  vnscene_t *scene,
  pokergameassets_t *assets
);

/**
 * Adjusts the camera to look at a specific player. You can use this to look at
 * the dealer by referencing the POKER_PLAYER_COUNT as the index.
 * 
 * @param scene Scene to adjust.
 * @param playerIndex Player index to look at.
 */
void pokerWorldLookAtPlayer(vnscene_t *scene, uint8_t playerIndex);

/**
 * Render the poker world.
 * 
 * @param world Poker Game World.
 * @param assets Assets to use.
 */
void pokerWorldRender(
  pokerworld_t *world, pokergameassets_t *assets
);

/**
 * Cleanup the poker world.
 * @param world World to clean up.
 */
void pokerWorldDispose(pokerworld_t *world);