/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "bet.h"

void pokerBetInit(pokerbet_t *bet) {
  bet->blindBig = POKER_BET_BLIND_BIG_DEFAULT;
  bet->blindSmall = POKER_BET_BLIND_SMALL_DEFAULT;
}

void pokerBetReset(pokerbet_t *bet) {
  bet->pot = 0;
  bet->currentBet = 0;
}

uint8_t pokerBetGetRoundPlayerDefault(uint8_t roundSmallBlind) {
  return (roundSmallBlind + 1) % POKER_PLAYER_COUNT;
}

void pokerBetPlayer(pokerbet_t *bet, pokerplayer_t *player, int32_t chips) {
  bet->pot += chips;
  player->chips -= chips;
  player->currentBet += chips;
  bet->currentBet = mathMax(bet->currentBet, player->currentBet);
}

void pokerBetResetBetter(
  pokerbet_t *poker, pokerplayer_t *players, uint8_t roundSmallBlind
) {
  uint8_t i;
  pokerplayer_t *player;
  for(i = 0; i < POKER_PLAYER_COUNT; i++) {
    player = players + i;
    player->state = flagOff(player->state, POKER_PLAYER_STATE_ROUND_MOVE);
  }
  bet->better = pokerBetGetRoundPlayerDefault(roundSmallBlind);
}

bool pokerBetPlayerCanBet(pokerbet_t *bet, pokerplayer_t *player) {
  if((player->state & POKER_PLAYER_STATE_FOLDED) != 0) return false;
  if(player->state & POKER_PLAYER_STATE_OUT) return false;
  if(player->state & POKER_PLAYER_STATE_ROUND_MOVE) {
    if(player->currentBet >= bet->currentBet && player) return false;
  }
  return true;
}

uint8_t pokerBetGetRemainingPlayer(
  pokerbet_t *bet, pokerplayer_t *players, uint8_t roundSmallBlind
) {
  uint8_t def, j, pi;
  def = pokerBetGetRoundPlayerDefault(roundSmallBlind);
  for(j = 0; j < POKER_PLAYER_COUNT; j++) {
    pi = ((def + j) % POKER_PLAYER_COUNT);
    if(!pokerBetPlayerCanBet(bet, players + pi)) continue;
    return pi;
  }
  return 0xFF;
}

void pokerBetTakeBlinds(
  pokerbet_t *bet, pokerplayer_t *players,
  uint8_t roundSmallBlind, uint8_t roundBigBlind
) {
  pokerBetPlayer(bet, players + roundSmallBlind, bet->blindSmall);
  pokerBetPlayer(bet, players + roundBigBlind, bet->blindBig);
}